;;;
;;; f o o . s c m		a few generic definitions for compatibility
;;;				with various Scheme implementations...
;;;

;(define foo$$documentation
"
foo.scm is a generic module to be loaded by specific compatibility files,
that helps with providing a uniform extension to the R4RS standard,
despite divergence in the way similar extensions are provided
by implementations.


Non-standard words and behaviors needed by Tunes:
(load (the string foo))
	should look up a load-path trying to load a file of name foo,
	which by default should contain . and $TUNES/src/HLL/scm
(add-load-path x)
	should add directory x to the load path
(provide (the symbol foo))
	should provide feature foo 
(require (the symbol foo))
	should load file foo.scm if feature foo is not provided yet.
(values x1 ... xn)
	is used to return multiple values to the continuation
(call-with-values (lambda () expr) (lambda (x1 ... xn) ...))
	is the only way Tunes will use to create a continuation
	to expression expr that will accept multiple return values.
(eval-now sexp)
	evaluates an expression in the current environment.
	Tunes will only use it at top-level
(format t string . stuff)
	slib-like text formatting

Optional words, used during debugging:
(sys$$getenv NAME)
	gets the value of environment variable $NAME
(sys$$system command)
	issues shell commands...
(sys$$exit)
	exits from Scheme to shell
(sys$$file-exists? name)
	checks for existence of a file of given name.
(sys$$reset)
	reset interpreter to pristine state
(sys$$error)
	system error reporting

Module foo.scm provides generic definitions for all of these,
with prefix foo$$. Some of the definitions require the specific
file to define a few words:
foo$$file-exists?	required if you use foo$$load for load-path capability


Macro-capability needed:
(define-macro% (macroname formals body...)
	defines a low-level macro that expands (lambda formals body...)
	then inserts the result as source in place of the macro call
	(which can itself be subjected to macro-expansion).
	Tunes will try not to rely upon the environment in which the lambda
	is expanded.

";)



;
; Generic definitions for 'require 'provide 'load
;
(define sys$$load load)

(define foo$$load-path (list "." *tunes-lib-path*))
(define (foo$$add-load-path x)
   (set! foo$$load-path (cons x foo$$load-path)))

(define (foo$$load x)
   (letrec
       ((try-path
	 (lambda (path)
	   (if (pair? path)
	       (let* ((dir  (car path))
		      (file (string-append dir "/" name)))
;		 (map display `("foo$$load$$try-path " ,x))(newline)
		 (if (sys$$file-exists? file)
		     (begin
;		       (map display `("Loading: " ,file "\n"))
		       (sys$$load file))
		     (try-path (cdr path))))
	       (sys$$load name))))
	(name (if (symbol? x) (symbol->string x) x)))
;     (map display `("foo$$load " ,x))(newline)
     (try-path foo$$load-path)))

(define foo$$features '())
(define (foo$$feature->filename feature)
   (string-append (symbol->string feature) ".scm"))

(define (foo$$require feature)
;   (map display `("Requiring feature " ,feature))
   (newline)
   (if (not (member feature foo$$features))
      (load (foo$$feature->filename feature))))

(define (foo$$provide feature)
   (set! foo$$features (cons feature foo$$features)))

;
; Generic definitions for values
;
(define (foo$$values . x) x)
(define (foo$$call-with-values vals cont)
  (apply cont (vals)))


;
; Poor man's evaluation
;
;might not work: load is not guaranteed to define at top-level...
(define foo$$counter
  (let ((n 0))
    (lambda () (let ((v n)) (set! n (+ v 1)) v))))
(define (make-new-tmp-filename . opt)
  (let ((prefix (if (pair? opt) (car opt) "/tmp/hlm-tmp-"))
	(suffix (if (and (pair? opt) (pair? (cdr opt))) (cadr opt) ".scm")))
    (string-append prefix (string->number (foo$$counter)) suffix)))
(define (foo$$eval sexp)
  (let* ((tf (make-new-tmp-filename))
	 (p  (open-output-file tf)))
    (if p
	(begin
	  (write sexp p)
	  (close-output-port p)
	  (load tf)))))

(define (foo$$reset)
  #f)

;(defmacro define-macro% (def . body)
;    `(defmacro ,(car def) ,(cdr def) ,@body))


(define (foo$$format t . args)
  (map (lambda (arg) (display arg) (newline)) (cons 'format args))
  #f)

